/* Copyright 2005 Pablo Guerrero Rosel <523958@unizar.es>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
#include <io.h> 
#include <console.h> 
 
unsigned short int console_buffer[WIDTH][HEIGHT];
int first_line;
int act_line;
int act_char;
 
char init_console()
{
	clr_console();
	
	first_line = 0;
	act_line = 0;
	act_char = 0;	
	
	return 0;
}
 
void put_char(char c)
{		
	if (c == '\n')
		new_line();
	else
	{
		console_buffer[act_char][(first_line + act_line)%HEIGHT] = c + CON_ATTRIB*0x100;
		
		act_char++;
		
		if (act_char == (WIDTH))
			new_line();
	}
	
}

void put_int(int num)
{
	char s[64],aux;
	int i,j;
	
	i=0;
	
	if (num<0)
	{
		num=-num;
		put_char('-');
	}
	
	do
	{
		s[i]='0'+(num%10);
		num/=10;
		i++;
	}while (num!=0);
	
	s[i]='\0';
	
	i--;
	
	for(j=0; j<(i/2)+1; j++)
	{
		aux=s[j];
		s[j]=s[i-j];
		s[i-j]=aux;
	}
	
	put_string(s);
	
}

void put_string(char * s)
{
	int i;
	
	for (i=0; s[i]!=0; i++)
		put_char(s[i]);
}	
	
void del_char()
{
	if (act_char>0)
		act_char--;
	else if (act_line>0)
	{
		act_line--;
		act_char = WIDTH - 1;
	}
	
	console_buffer[act_char][(first_line + act_line)%HEIGHT] = 0x20 + CON_ATTRIB*0x100;
}	
	
void clr_line()
{
	int i;
	unsigned short int space;
	
	space = 0x20 + CON_ATTRIB*0x100;
	
	for (i=0; i<WIDTH; i++)
			console_buffer[i][(first_line + act_line)%HEIGHT]=space;		
		
}

void clr_console()
{
	int i,j;
	unsigned short int space;
	
	space = 0x20 + CON_ATTRIB*0x100;
	
	for (i=0; i<HEIGHT; i++)
		for (j=0; j<WIDTH; j++)
			console_buffer[j][i]=space;		
		
}
	
void new_line()
{	
	act_char = 0;
	
	if(act_line == (HEIGHT-1))
		first_line = (first_line+1)%HEIGHT;	
	else
		act_line++;
	
	clr_line();
}

// Copy console_buffer to VID_MEM
void refresh_console()
{
	int i,j;
	
	unsigned short int * vid_mem;
	
	for (i=0; i<HEIGHT; i++)
		for (j=0; j<WIDTH; j++)
		{
			vid_mem =(unsigned short int *)(VID_MEM + ((i*WIDTH) + j)*2);
			*vid_mem = console_buffer[j][(first_line+i)%HEIGHT];
		}
}

void update_cursor()
{

	unsigned short	position;
	
	position=(act_line*80) + act_char; // Position on the console
		
	outb(0x3D4, 0x0F);
	outb(0x3D5, (unsigned char)(position&0xFF));
	outb(0x3D4, 0x0E);
	outb(0x3D5, (unsigned char)((position>>8)&0xFF));
}
