/* Copyright 2005 Pablo Guerrero Rosel <523958@unizar.es>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
#ifndef __gdt_h__
#define __gdt_h__


#define GDT_BASE 0x6000 //Base GDT address 
#define GDT_BASE_L 0x6000
#define GDT_BASE_H 0x0000

//Type 
#define GDT_DAT_R 0x0 // Data Read
#define GDT_DAT_RW 0x2 // DATA Read/Write
#define GDT_STK_R 0x4 // Stack Read
#define GDT_STK_RW 0x6 // Stack Read/Write
#define GDT_COD_E 0x8 // Code Execute
#define GDT_COD_ER 0xA //Code Execute/Read
#define GDT_COD_EC 0xC //Code Execute Conforming (without privilege protection)
#define GDT_COD_ERC 0xE //Code Execute/Read Conforming (without privilege protection)

// Segment Type
#define GDT_SYSTEM 0x0 // System descriptor
#define GDT_CODE_DATA 0x1 // Code, Data or Stack descriptor

// Privilege Level
#define GDT_RING_0 0x0 // More privilege (kernel)
#define GDT_RING_1 0x1
#define GDT_RING_2 0x2
#define GDT_RING_3 0x3 // Less Privilege (user)

// Present
#define GDT_NOT_PRESENT 0x0 // Undefined rescriptor
#define GDT_PRESENT 0x1 // Valid descriptor

// Attributes (Default GDT_SIZE_32|GDT_GRAN_PAGES = 0xC)
// Size of address
#define GDT_SIZE_16 0x0 // 16 bits Instrucctions, Offsets and Registers 
#define GDT_SIZE_32 0x4 // 32 bits Instrucctions, Offsets and Registers 
// Granularity in bytes or pages (4Kb)
#define GDT_GRAN_BYTES 0x0 // Segment sizes 1B to 1MB
#define GDT_GRAN_PAGES 0x8 // Segment sizes 4KB to 4GB

// GDT descriptor entry
typedef struct gdt_entry_struct
{
	unsigned short int seg_limit_l;       // Segment Limit Low (0-15)
	unsigned short int base_l;		// Base Address Low (0-15)
	unsigned char base_m;			// Base Address Middle (16-23)
	unsigned char type:4;			// Type
	unsigned char seg_type:1;		// Segment Type (SYSTEM or APLICATION)
	unsigned char p_level:2;				// Privilege level
	unsigned char present:1;			// Present
	unsigned char seg_limit_h:4;		// Segment Limit High (16-19)
	unsigned char attrib:4;		// Attributes
	unsigned char base_h;		// Base Address High (24-31)
}tp_gdt_entry;

// GDT register
typedef struct gdt_register_struct
{
	unsigned short int length; // GDT length
	unsigned short int base_l; // GDT Base Address Low (0-15) 
	unsigned short int base_h; // GDT Base Address High (16-31)
}tp_gdtr;

int init_gdt();
void load_gdt_entry(tp_gdt_entry *gdt_entry, int gdt_num);
void load_gdtr();

#endif
