/* Copyright 2005 Pablo Guerrero Rosel <523958@unizar.es>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
#include <idt.h>
#include <io.h>
#include <pic.h> 
#include <timer.h>
#include <console.h>
#include <floppy.h>

static long time_elapsed = 0; // Tick of timer_handler

void init_timer ()
{

	#define LATCH  (PIT_FREQUENCY/INTERVAL)
	
	// Set the frequency
	outb(PIT_MODE,0x36);	// Program counter0 (system)	
	outb(PIT,LATCH & 0xFF);	// Low byte (0-7)
	outb(PIT,LATCH >> 8);	 // High byte (8-15)
	
	// Load the IDT timer handler 
	load_idt_entry(32,(unsigned int)&timer_handler,IDT_CS_SELECTOR,
	IDT_INT_GATE|IDT_SIZE_32|IDT_RING_0|IDT_PRESENT);

	// Enable IRQ of system timer
	enable_irq(0);
	
	put_string("Timer Initialized [ DONE ]\n");
}	

void timer_handler()
{
	if (time_elapsed%20 == 0) // Max 5 strokes per second
		time_fd_handler(); // Call floppy time handler
	
	refresh_console(); // Refresh the console
	update_cursor();
	
	time_elapsed++; // Tick of timer_handler +=1
	
	outb(0x20, 0x20);
	
	asm("mov %ebp,%esp");
	asm("pop %ebp");
	asm("iret");
}

void sleep(int msec)
{
	long act_time = time_elapsed;
	
	while (time_elapsed < (act_time + (msec/10)));
	
}
